from __future__ import annotations

import argparse
import dataclasses

from ldaptool._utils import argclasses


def _parser_add_attributes(parser: argparse.ArgumentParser, dest: str) -> None:
    parser.add_argument(
        metavar="attributes",
        dest=dest,
        nargs="*",
        help="""
            Attributes to lookup (and columns to display in tables).
            Fake attributes `dndomain`, `dnpath` an `dnfullpath` are available (created from dn).
        """,
    )


@dataclasses.dataclass(slots=True, kw_only=True)
class Arguments(argclasses.BaseArguments):
    columns: list[str] = dataclasses.field(default_factory=list, metadata=argclasses.manual(_parser_add_attributes))
    attributes: list[str] = dataclasses.field(default_factory=list)

    json: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(help="Use full json output"),
    )
    human: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(help="Use simple json output (join multiple values of one attribute)"),
    )
    human_separator: str = dataclasses.field(
        default=", ",
        metadata=argclasses.arg(help="Separator to join multiple values of one attribute with (default: %(default)r)"),
    )

    dateonly: bool = dataclasses.field(
        default=True,
        metadata=argclasses.arg(help="Use only date part of decoded timestamps"),
    )

    dndomain: bool = False
    dnpath: bool = False
    dnfullpath: bool = False

    def __post_init__(self) -> None:
        super(Arguments, self).__post_init__()  # super() not working here, unclear why.

        # extract special attribute names
        attributes_set: dict[str, str] = {arg.lower(): arg for arg in self.columns}  # index by lowercase name
        # create fake attributes on demand
        if attributes_set.pop("dndomain", ""):
            self.dndomain = True
        if attributes_set.pop("dnpath", ""):
            self.dnpath = True
        if attributes_set.pop("dnfullpath", ""):
            self.dnfullpath = True
        # store remaining attributes (with original case)
        self.attributes = list(attributes_set.values())
        if self.columns and not self.attributes:
            # if we only wanted fake attributes, make sure we only request 'dn' - empty list would query all attributes
            self.attributes = ["dn"]
